# -*- coding: utf-8 -*-
"""
Created on Mon Jul 17 16:36:17 2017

@author: Maximilian Edich

iGEM Bielefeld-CeBiTec 2017
"""

#import modules
import sys
import re
import os
import datetime

"""
Takes a string and tries to convert it into a number. If this operation is
successful it return true, otherwise false.
"""
def isNumber(string):
    try:
        float(string)
        return True
    except ValueError:
        return False

"""
Takes two dates in the format YYYY-MM-DD and compares them. Return 1, if the
first date is before the second date. Otherwise it returns 0.
"""
def dateIsSmaller(date1, date2):
    year1 = float(date1[0:4])
    month1 = float(date1[5:7])
    day1 = float(date1[8:10])
    year2 = float(date2[0:4])
    month2 = float(date2[5:7])
    day2 = float(date2[8:10])
    
    if (year1 < year2):
        return 1
    elif (year1 == year2):
        if (month1 < month2):
            return 1
        elif (month1 == month2):
            if (day1 < day2):
                return 1
    return 0

#set index variables to access data of entries
INDEX_DATE = 0
INDEX_TEAM = 1
INDEX_TITLE = 4
INDEX_INVEST = 3
INDEX_EXPERIMENT = 2
INDEX_PROCEDURE = 5

#check input
if (len(sys.argv) < 2):
    sys.exit("File input is missing!\n\nUsage:\tpython labnotes_generator.py \
<input file/folder> -top <html top insert> -bottom <html bottom insert> \
-names <names dict> -protocols <protocols dict>\n\n\
write\npython labnotes_generator.py -help\n to get a list of all parameters")


print("Labnotes Generator\tVersion: 1.0")
print("Bielefeld-CeBiTec 2017\n")


if (sys.argv[1] == "-h" or sys.argv[1] == "-help"):
    print("labnotes_generator.py <input file>\timports text file in labnote format\n")
    print("labnotes_generator.py <input folder>\timports all files in folder\n")
    print("-top <html top insert>\tinserts html code before the labjournal code\n")
    print("-bottom <html bottom insert>\tinserts html code after the labjournal code\n")
    print("-names <names dict>\t\load dictionary to replaced initials by full names\n")
    print("-protocols <protocols dict>\tload dictionary to connect keywords with protocols\n")
    print("-nocss\tsuppress creation of default css file\n")
    print("-nojs\tsuppress creation of default js file\n")
    print("-out <path/folder name>\tgenerate output folder for all files\n")


#create variables editable by optional parameters
htmlTop = ""
htmlBottom = ""
namesFile = ""
protocolsFile = ""
writeCSS = 1
writeJS = 1
outPath = ""

#look for optional parameters and load input
for x in range(len(sys.argv[1:])):
    arg = sys.argv[x+1]
    if (arg == "-top" and x+2 <= len(sys.argv[1:])):
        x += 1
        file = open(sys.argv[x+1], 'r')
        htmlTop = file.read()
        file.close()
    elif (arg == "-bottom" and x+2 <= len(sys.argv[1:])):
        x += 1
        file = open(sys.argv[x+1], 'r')
        htmlBottom = file.read()
        file.close()
    elif (arg == "-names" and x+2 <= len(sys.argv[1:])):
        x += 1
        namesFile = sys.argv[x+1]
    elif (arg == "-protocols" and x+2 <= len(sys.argv[1:])):
        x += 1
        protocolsFile = sys.argv[x+1]
    elif (arg == "-nocss"):
        writeCSS = 0
    elif (arg == "-nojs"):
        writeCSS = 0
    elif (arg == "-out" and x+2 <= len(sys.argv[1:])):
        x += 1
        outPath = sys.argv[x+1] + "\\"

#load the name substitution dictonary
shortNames = {}
if (namesFile != ""):
    with open(namesFile,'r') as inf:
        shortNames = eval(inf.read())

#load the protocol dictionary
protocols = {}
if (protocolsFile != ""):
    with open(protocolsFile,'r') as inf:
        protocols = eval(inf.read())
protocolKeys = list(protocols.keys())

#open input file or directory
if (os.path.isfile(sys.argv[1])):
    #input is single file
    file = open(sys.argv[1], 'r')
    text = file.readlines()
    file.close()
elif (os.path.isdir(sys.argv[1])):
    #input is folder, load all files
    text = []
    files = os.listdir(sys.argv[1])
    for fileName in files:
        file = open(sys.argv[1] + "\\" + fileName, 'r')
        newText = file.readlines()
        file.close()
        for line in newText:
            text.append(line)
        text.append("\n")
else:
    print("ERROR: Input is not a file or directory!")
    sys.exit("\nUsage:\tpython labnotes_creator.py \
<input file/folder> -top <html top insert> -bottom <html bottom insert> \
-names <names dict> -protocols <protocols dict>\n")


#get information from input
procedure = []
entry = []
entries = []
saveEntry = 0
print()
if (text[0] != "\n" and text[0].find(">") > -1):
    text[0] = text[0][text[0].find(">"):]
for line in text:
    if (saveEntry > 0):
        #save the lines until an empty line appears
        if (line.strip() == ""):
            if (len(entry) > 0):
                saveEntry = 0
                entry.append(procedure)
                entries.append(entry)
                entry = []
                procedure = []
        elif (saveEntry >= INDEX_PROCEDURE):
            #save procedure
            newLine = line.strip()
            for key in protocolKeys:
                pos = newLine.lower().find(key)
                if (pos > 0):
                    newLine = newLine[0:pos] + protocols.get(key) + newLine[(pos + len(key)):]
                    break
            procedure.append(newLine)
        else:
            #save team, title, investigators and aim
            if (saveEntry == INDEX_INVEST):
                #
                investigators = re.split(" ", line.strip())
                if (shortNames.get(investigators[0]) != None):
                    fullNames = shortNames.get(investigators[0])
                else:
                    fullNames = investigators[0]
                for name in investigators[1:]:
                    if (shortNames.get(name) != None):
                        fullNames = fullNames + ", " + str(shortNames.get(name))
                    else:
                        fullNames = str(fullNames) + ", " + str(name)
                entry.append(fullNames)
            else:
                #
                entry.append(line.strip())
            saveEntry += 1
    else:
        #search for ">"
        if (line[0] == '>'):
            saveEntry = 1
            #getDate and check date formats
            date = line.strip()[1:].strip()
            if (date.find(".") > 0):
                day = date[0:2]
                month = date[3:5]
                year = date[6:11]
                newDate = year + "-" + month + "-" + day
                entry.append(newDate)
            elif (date.find("/") > 0):
                month = date[0:2]
                day = date[3:5]
                year = date[6:11]
                newDate = year + "-" + month + "-" + day
                entry.append(newDate)
            elif (date.find("-") > 0):
                entry.append(date)
            else:
                sys.exit("Unknown date format! Use one of these formats:\n\
DD.MM.YYYY\nYYYY-MM-DD\nMM/DD/YYYY")
            
#add last entry
if (len(entry) > 0):
    entry.append(procedure)
    entries.append(entry)
entry = []
procedure = []

#create date groups
dateGroups = []
for entry in entries:
    added = 0
    date = entry[INDEX_DATE]
    for dateGroup in dateGroups:
        if (dateGroup[0][INDEX_DATE] == date):
            dateGroup.append(entry)
            added = 1
            break
    if (added == 0):
        #filter out wrong dates
        if (isNumber(entry[INDEX_DATE][0:4]) and
            isNumber(entry[INDEX_DATE][5:7]) and
            isNumber(entry[INDEX_DATE][8:10])):
            newDateGroup = [entry]
            dateGroups.append(newDateGroup)

#sort date groups
newDateGroups = []
for dateGroup in dateGroups:
    if (len(newDateGroups) == 0):
        newDateGroups.append(dateGroup)
    else:
        added = 0
        pos = -1
        for ndg in newDateGroups:
            pos += 1
            if (dateIsSmaller(dateGroup[0][INDEX_DATE], ndg[0][INDEX_DATE])):
                half1 = newDateGroups[0:pos]
                half2 = newDateGroups[pos:]
                newDateGroups = half1 + [dateGroup] + half2
                added = 1
                break
        if (added == 0):
            newDateGroups.append(dateGroup)

#substitute dates by time span of a week
for dateGroup in newDateGroups:
    for entry in dateGroup:
        try:
            date = datetime.date(int(entry[INDEX_DATE][0:4]), int(entry[INDEX_DATE][5:7]), int(entry[INDEX_DATE][8:10]))
        except ValueError:
            sys.exit("ERROR: Wrong date format! The wrong date is: " + entry[INDEX_DATE])
        start = date - datetime.timedelta(days=date.weekday())
        end = start + datetime.timedelta(days=6)
        entry[INDEX_DATE] = str(start) + "   -   " + str(end)

#group projects in the same week together
dateGroups = newDateGroups
newDateGroups = []
for dateGroup in dateGroups:
    if (len(newDateGroups) == 0):
        newDateGroups.append(dateGroup)
    else:
        added = 0
        for ndg in newDateGroups:
            if (ndg[0][INDEX_DATE] == dateGroup[0][INDEX_DATE]):
                added = 1
                for entry in dateGroup:
                    ndg.append(entry)
                break
        if (added == 0):
            newDateGroups.append(dateGroup)

#print out weeks
for ndg in newDateGroups:
    print(ndg[0][0])

#save changes in dategroups
dateGroups = newDateGroups

#create html code
htmlCodeMiddle = ["<!--generated by the Labnotesgenerator from Bielefeld-CeBiTec 2017-->\n<meta charset=\"UTF-8\">"]
if (writeCSS == 1):
    htmlCodeMiddle.append("<link rel=\"stylesheet\" type=\"text/css\" href=\"labnotes_css.css\">\n")
if (writeJS == 1):
    htmlCodeMiddle.append("<script src=\"https://ajax.googleapis.com/ajax/libs/jquery/3.2.1/jquery.min.js\"></script>\n<script src=\"labnotes_js.js\"></script>\n")
for dateGroup in dateGroups:
    if (len(htmlCodeMiddle) == 0):
        #first entry has margin
        htmlCodeMiddle.append('            <div class="labnotebox" style="margin-top: 50px;">\n\
				<div class="labnote-date">\n\
					<h3> ' + dateGroup[0][INDEX_DATE] + ' </h3>\n\
				</div>\n\
			</div>\n')
    else:
        htmlCodeMiddle.append('            <div class="labnotebox">\n\
				<div class="labnote-date">\n\
					<h3> ' + dateGroup[0][INDEX_DATE] + ' </h3>\n\
				</div>\n\
			</div>\n')
    
    for entry in dateGroup:
        htmlCodeMiddle.append('			<div class="labnotebox">\n\
				<div class="labnote-title">\n\
					<h3> ' + entry[INDEX_TITLE] + ' </h3>\n\
				</div>\n\
				<div class="labnote-content">\n')
        if (str(entry[INDEX_TEAM]) != "?" and str(entry[INDEX_TEAM])!= "???"):
            htmlCodeMiddle.append('\
               <p><b>Team:</b> ' + str(entry[INDEX_TEAM]) + '</p>\n')
        if (str(entry[INDEX_INVEST]) != "?" and str(entry[INDEX_INVEST])!= "???"):
            htmlCodeMiddle.append('\
					<p><b>Investigators:</b> ' + str(entry[INDEX_INVEST]) + '</p>\n')
        if (str(entry[INDEX_EXPERIMENT]) != "?" and str(entry[INDEX_EXPERIMENT])!= "???"):
            htmlCodeMiddle.append('\
					<p><b>Superior experiment:</b> ' + str(entry[INDEX_EXPERIMENT]) + '</p>\n')
        htmlCodeMiddle.append('\
					<p><b>Procedure:</b></p>\n\
					<article>\n')
        sublevel = 0
        for step in entry[INDEX_PROCEDURE]:
            #count minuses
            minusCount = 0
            for char in step:
                if (char == '-'):
                    minusCount += 1
                else:
                    break
            if (minusCount > sublevel):
                htmlCodeMiddle.append('<ul>\n')
                sublevel += 1
            elif (minusCount < sublevel):
                htmlCodeMiddle.append('</ul>\n')
                sublevel -= 1
            htmlCodeMiddle.append('<li> ' + step[minusCount:] + '</li>\n')
        htmlCodeMiddle.append('</ul>\n')
                
        htmlCodeMiddle.append('</article>\n\
				</div>\n\
			</div>\n')


#join code pieces
htmlCode = ''.join([htmlTop, ''.join(htmlCodeMiddle), htmlBottom])

#check if outputfolder exists and make new one if needed
if (outPath != ""):
    if (not os.path.exists(outPath)):
        os.makedirs(outPath)

#write to final file
file = open(outPath + "labnotes.html", 'w')
file.write(htmlCode)
file.close()

#write java script
if (writeJS == 1):
    jsCode = "$(document).ready(function() {\
    $(\".labnote-title\").click(function() {\
        $title = $(this);\
		$content = $title.next();\
		if (!$content.is(\":visible\")) {\
			$title.css(\"border-bottom-right-radius\", \"0px\");\
			$title.css(\"border-bottom-left-radius\", \"0px\");\
		}\
		$content.slideToggle(500, function() {\
			if (!$content.is(\":visible\")) {\
				$title.css(\"border-radius\", \"10px\");\
				$title.css(\"zIndex\", \"0\");\
			}\
		});\
    });\
});"
    file = open(outPath + "labnotes_js.js", 'w')
    file.write(jsCode)
    file.close()

#write css
if (writeCSS == 1):
    cssCode = "/*the area of the box on full width*/\
.labnotebox {\
	width: 100%;\
	margin: auto;\
	background-color: #ffffff;\
}\
\
/*the box itself with the content*/\
.labnote-content {\
	width: 50%;\
	overflow: auto;\
	margin: auto;\
	padding-bottom: 20px;\
	display: none;\
	background-color: #dddddd;\
	border-bottom-left-radius: 10px;\
	border-bottom-right-radius: 10px;\
	padding-left: 50px;\
	padding-right: 50px;\
}\
\
/*the clickable title*/\
.labnote-title {\
	padding-bottom: 5px;\
	padding-left: 50px;\
	padding-right: 50px;\
	padding-top: 5px;\
	text-align: justify;\
	margin-top: 20px;\
	margin: auto;\
	cursor: pointer;\
	text-align: center;\
	background-color: #dddddd;\
	width: 50%;\
	margin-top: 20px;\
	border-radius: 10px;\
}\
\
/*the date*/\
.labnote-date {\
	padding: 1px;\
	padding-left: 50px;\
	padding-right: 50px;\
	padding-top: 5px;\
	padding-bottom: 5px;\
	text-align: center;\
	margin: auto;\
	margin-top: 20px;\
	margin-bottom: 20px;\
	background-color: #eeeeee;\
	width: 50%;\
	border-radius: 10px;\
}"
    file = open(outPath + "labnotes_css.css", 'w')
    file.write(cssCode)
    file.close()
